#include <stdio.h>

static const char *TAG = "USB";
#include "esp_log.h"

#include "tinyusb.h"
#include "tusb_cdc_acm.h"

#include "my_usb.h"

static uint8_t buf[CONFIG_TINYUSB_CDC_RX_BUFSIZE + 1];

void tinyusb_cdc_rx_callback(int itf, cdcacm_event_t *event)
{
  /* initialization */
  size_t rx_size = 0;

  /* read */
  esp_err_t ret = tinyusb_cdcacm_read( itf, buf, CONFIG_TINYUSB_CDC_RX_BUFSIZE, &rx_size);
  if (ret == ESP_OK) {
    ESP_LOGI(TAG, "Data from channel %d:", itf);
    ESP_LOG_BUFFER_HEXDUMP(TAG, buf, rx_size, ESP_LOG_INFO);
  } else {
    ESP_LOGE(TAG, "Read error");
  }

  /* write back */
  tinyusb_cdcacm_write_queue(1-itf, buf, rx_size);
  tinyusb_cdcacm_write_flush(1-itf, 0);
}

void tinyusb_cdc_line_state_changed_callback(int itf, cdcacm_event_t *event)
{
  int dtr = event->line_state_changed_data.dtr;
  int rts = event->line_state_changed_data.rts;
  ESP_LOGI(TAG, "Line state changed on channel %d: DTR:%d, RTS:%d", itf, dtr, rts);
}


void usb_init(void)
{
  tinyusb_config_t const tusb_cfg = {
    .device_descriptor = NULL,
    .string_descriptor = NULL,
    .external_phy = false,
    .configuration_descriptor = NULL,
  };

  tinyusb_config_cdcacm_t acm_cfg = {
    .usb_dev = TINYUSB_USBDEV_0,
    .rx_unread_buf_sz = 64,
    .callback_rx = &tinyusb_cdc_rx_callback,
    .callback_rx_wanted_char = NULL,
    .callback_line_state_changed = tinyusb_cdc_line_state_changed_callback,
    .callback_line_coding_changed = NULL
  };

  ESP_LOGI(TAG, "USB initialization");
  ESP_ERROR_CHECK( tinyusb_driver_install(&tusb_cfg) );

  acm_cfg.cdc_port = TINYUSB_CDC_ACM_0;
  ESP_ERROR_CHECK(tusb_cdc_acm_init(&acm_cfg));

  acm_cfg.cdc_port = TINYUSB_CDC_ACM_1;
  ESP_ERROR_CHECK( tusb_cdc_acm_init(&acm_cfg) );

  ESP_LOGI(TAG, "USB initialization DONE");
}
